@@ -4,7 +4,8 @@ from django.contrib import admin, messages
 from django.contrib.admin.options import BaseModelAdmin, InlineModelAdmin
 from django.core.exceptions import PermissionDenied
 from django.http import HttpResponseRedirect
-from django.urls import reverse
+from django.shortcuts import redirect
+from django.urls import reverse, reverse_lazy, path
 from django.utils.safestring import mark_safe
 
 from .admin_utils import remove_from_fieldsets
@@ -110,6 +111,12 @@ class EnhancedAdminMixin(admin.ModelAdmin, EnhancedBaseAdminMixin):
         (url, dict(link_text, link_class, new_tab, user_passes_test))
     """
     extra_changelist_links = []
+    """
+        List of tuples:
+        ('name', dict(btn_text, btn_class, callback, short_desc))
+        Note: callback is called with args (request, instance)
+    """
+    action_links = []
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -129,6 +136,22 @@ class EnhancedAdminMixin(admin.ModelAdmin, EnhancedBaseAdminMixin):
             func.__name__ = f[0]
             setattr(self, func.__name__, func)
 
+        for name, attrs in self.action_links:
+            if 'callback' not in attrs:
+                continue
+            func = lambda instance: mark_safe(
+                '<a class="btn {0}" href="{1}">{2}</a>'.format(
+                    attrs.get('btn_class', ''),
+                    reverse(f'admin:{self.model._meta.app_label}_{self.model._meta.model_name}_action_view_{name}',
+                            args=(instance.id,)),
+                    attrs.get('btn_text', ''),
+                ))
+            func.allow_tags = True
+            if 'short_desc' in attrs:
+                func.short_description = attrs['short_desc']
+            func.__name__ = name
+            setattr(self, func.__name__, func)
+
     def get_search_fields(self, request):
         search_fields = deepcopy(self.search_fields)
         if self.search_in_choices:
@@ -233,7 +256,9 @@ class EnhancedAdminMixin(admin.ModelAdmin, EnhancedBaseAdminMixin):
                 inline_instances.remove(inline)
 
     def get_readonly_fields(self, request, obj=None):
-        return list(super().get_readonly_fields(request, obj)) + [x[0] for x in self.custom_fields]
+        return (list(super().get_readonly_fields(request, obj)) +
+                [x[0] for x in self.custom_fields] +
+                [x[0] for x in self.action_links])
 
     def get_extra_changelist_links(self, request):
         return self.extra_changelist_links
@@ -257,6 +282,25 @@ class EnhancedAdminMixin(admin.ModelAdmin, EnhancedBaseAdminMixin):
             remove_from_fieldsets(fieldsets, field_exclusions)
         return fieldsets
 
+    def get_urls(self):
+        urls = super().get_urls()
+        my_urls = []
+
+        for (name, attrs) in self.action_links:
+            if 'callback' not in attrs:
+                continue
+            view_name = f'{self.model._meta.app_label}_{self.model._meta.model_name}_action_view_{name}'
+
+            def action_link_view(request, pk):
+                obj = self.model.objects.get(id=pk)
+                attrs['callback'](obj)
+                return redirect(request.META.get('HTTP_REFERER'))
+
+            action_link_view.__name__ = view_name
+            my_urls.append(path(f'<path:pk>/action-link/{name}/', self.admin_site.admin_view(action_link_view),
+                                name=view_name))
+        return my_urls + urls
+
 
 class ReadonlyAdmin(admin.ModelAdmin):
 
